@@ -59,15 +59,12 @@ class TestProjectStructure:
         """
         # The test below had a but for quite a while and we missed a lot of modules to have tess
         # We should make sure that one goes to 0
-        # TODO(potiuk) - check if that test actually tests something
         OVERLOOKED_TESTS = [
             "providers/amazon/tests/unit/amazon/aws/auth_manager/datamodels/test_login.py",
             "providers/amazon/tests/unit/amazon/aws/auth_manager/security_manager/test_aws_security_manager_override.py",
             "providers/amazon/tests/unit/amazon/aws/executors/batch/test_batch_executor_config.py",
             "providers/amazon/tests/unit/amazon/aws/executors/batch/test_boto_schema.py",
-            "providers/amazon/tests/unit/amazon/aws/executors/batch/test_utils.py",
             "providers/amazon/tests/unit/amazon/aws/executors/ecs/test_ecs_executor_config.py",
-            "providers/amazon/tests/unit/amazon/aws/executors/ecs/test_utils.py",
             "providers/amazon/tests/unit/amazon/aws/executors/aws_lambda/test_utils.py",
             "providers/amazon/tests/unit/amazon/aws/executors/aws_lambda/docker/test_app.py",
             "providers/amazon/tests/unit/amazon/aws/executors/utils/test_base_config_keys.py",
@@ -82,7 +79,6 @@ class TestProjectStructure:
             "providers/amazon/tests/unit/amazon/aws/utils/test_sagemaker.py",
             "providers/amazon/tests/unit/amazon/aws/waiters/test_base_waiter.py",
             "providers/apache/hdfs/tests/unit/apache/hdfs/hooks/test_hdfs.py",
-            "providers/apache/hdfs/tests/unit/apache/hdfs/log/test_hdfs_task_handler.py",
             "providers/apache/hdfs/tests/unit/apache/hdfs/sensors/test_hdfs.py",
             "providers/apache/hive/tests/unit/apache/hive/plugins/test_hive.py",
             "providers/celery/tests/unit/celery/executors/test_celery_executor_utils.py",
@@ -112,7 +108,6 @@ class TestProjectStructure:
             "providers/edge3/tests/unit/edge3/models/test_edge_worker.py",
             "providers/edge3/tests/unit/edge3/worker_api/routes/test__v2_compat.py",
             "providers/edge3/tests/unit/edge3/worker_api/routes/test__v2_routes.py",
-            "providers/edge3/tests/unit/edge3/worker_api/routes/test_jobs.py",
             "providers/edge3/tests/unit/edge3/worker_api/test_app.py",
             "providers/edge3/tests/unit/edge3/worker_api/test_auth.py",
             "providers/edge3/tests/unit/edge3/worker_api/test_datamodels.py",
@@ -137,10 +132,8 @@ class TestProjectStructure:
             "providers/fab/tests/unit/fab/www/test_security_appless.py",
             "providers/fab/tests/unit/fab/www/test_security_manager.py",
             "providers/fab/tests/unit/fab/www/test_session.py",
-            "providers/fab/tests/unit/fab/www/test_utils.py",
             "providers/fab/tests/unit/fab/www/test_views.py",
             "providers/google/tests/unit/google/cloud/fs/test_gcs.py",
-            "providers/google/tests/unit/google/cloud/links/test_automl.py",
             "providers/google/tests/unit/google/cloud/links/test_base.py",
             "providers/google/tests/unit/google/cloud/links/test_bigquery.py",
             "providers/google/tests/unit/google/cloud/links/test_bigquery_dts.py",
@@ -165,8 +158,8 @@ class TestProjectStructure:
             "providers/google/tests/unit/google/cloud/links/test_pubsub.py",
             "providers/google/tests/unit/google/cloud/links/test_spanner.py",
             "providers/google/tests/unit/google/cloud/links/test_stackdriver.py",
-            "providers/google/tests/unit/google/cloud/links/test_vertex_ai.py",
             "providers/google/tests/unit/google/cloud/links/test_workflows.py",
+            "providers/google/tests/unit/google/cloud/links/test_translate.py",
             "providers/google/tests/unit/google/cloud/operators/vertex_ai/test_auto_ml.py",
             "providers/google/tests/unit/google/cloud/operators/vertex_ai/test_batch_prediction_job.py",
             "providers/google/tests/unit/google/cloud/operators/vertex_ai/test_custom_job.py",
@@ -181,9 +174,6 @@ class TestProjectStructure:
             "providers/google/tests/unit/google/cloud/transfers/test_presto_to_gcs.py",
             "providers/google/tests/unit/google/cloud/utils/test_bigquery.py",
             "providers/google/tests/unit/google/cloud/utils/test_bigquery_get_data.py",
-            "providers/google/tests/unit/google/cloud/utils/test_dataform.py",
-            "providers/google/tests/unit/google/common/links/test_storage.py",
-            "providers/google/tests/unit/google/common/test_consts.py",
             "providers/google/tests/unit/google/common/hooks/test_operation_helpers.py",
             "providers/google/tests/unit/google/test_go_module_utils.py",
             "providers/http/tests/unit/http/test_exceptions.py",
@@ -193,10 +183,7 @@ class TestProjectStructure:
             "providers/standard/tests/unit/standard/operators/test_branch.py",
             "providers/standard/tests/unit/standard/operators/test_empty.py",
             "providers/standard/tests/unit/standard/operators/test_latest_only.py",
-            "providers/standard/tests/unit/standard/operators/test_trigger_dagrun.py",
             "providers/standard/tests/unit/standard/sensors/test_external_task.py",
-            "providers/standard/tests/unit/standard/sensors/test_filesystem.py",
-            "providers/standard/tests/unit/standard/utils/test_sensor_helper.py",
             "providers/sftp/tests/unit/sftp/test_exceptions.py",
         ]
         modules_files: list[pathlib.Path] = list(
@@ -439,6 +426,7 @@ class TestGoogleProviderProjectStructure(ExampleCoverageTest, AssetsCoverageTest
         "airflow.providers.google.cloud.operators.automl.AutoMLDeleteModelOperator",
         "airflow.providers.google.cloud.operators.automl.AutoMLListDatasetOperator",
         "airflow.providers.google.cloud.operators.automl.AutoMLDeleteDatasetOperator",
+        "airflow.providers.google.cloud.operators.vertex_ai.auto_ml.CreateAutoMLVideoTrainingJobOperator",
         "airflow.providers.google.cloud.operators.bigquery.BigQueryCreateEmptyTableOperator",
         "airflow.providers.google.cloud.operators.bigquery.BigQueryCreateExternalTableOperator",
         "airflow.providers.google.cloud.operators.datapipeline.CreateDataPipelineOperator",
@@ -477,6 +465,12 @@ class TestGoogleProviderProjectStructure(ExampleCoverageTest, AssetsCoverageTest
         "airflow.providers.google.cloud.operators.datacatalog.CloudDataCatalogUpdateTagTemplateOperator",
         "airflow.providers.google.cloud.operators.datacatalog.CloudDataCatalogCreateEntryOperator",
         "airflow.providers.google.cloud.operators.datacatalog.CloudDataCatalogUpdateTagTemplateFieldOperator",
+        "airflow.providers.google.cloud.operators.vertex_ai.generative_model.GenerateFromCachedContentOperator",
+        "airflow.providers.google.cloud.operators.vertex_ai.generative_model.CreateCachedContentOperator",
+        "airflow.providers.google.cloud.operators.vertex_ai.generative_model.CountTokensOperator",
+        "airflow.providers.google.cloud.operators.vertex_ai.generative_model.SupervisedFineTuningTrainOperator",
+        "airflow.providers.google.cloud.operators.vertex_ai.generative_model.GenerativeModelGenerateContentOperator",
+        "airflow.providers.google.cloud.operators.vertex_ai.generative_model.TextEmbeddingModelGetEmbeddingsOperator",
     }
 
     BASE_CLASSES = {
@@ -505,13 +499,6 @@ class TestGoogleProviderProjectStructure(ExampleCoverageTest, AssetsCoverageTest
         "airflow.providers.google.cloud.operators.vertex_ai.endpoint_service.UpdateEndpointOperator",
         "airflow.providers.google.cloud.operators.vertex_ai.batch_prediction_job.GetBatchPredictionJobOperator",
         "airflow.providers.google.cloud.operators.datacatalog.CloudDataCatalogDeleteEntryOperator",
-        "airflow.providers.google.marketing_platform.operators.display_video.GoogleDisplayVideo360CreateQueryOperator",
-        "airflow.providers.google.marketing_platform.operators.display_video.GoogleDisplayVideo360RunQueryOperator",
-        "airflow.providers.google.marketing_platform.operators.display_video.GoogleDisplayVideo360DeleteReportOperator",
-        "airflow.providers.google.marketing_platform.operators.display_video.GoogleDisplayVideo360DownloadReportV2Operator",
-        "airflow.providers.google.marketing_platform.operators.display_video.GoogleDisplayVideo360DownloadLineItemsOperator",
-        "airflow.providers.google.marketing_platform.operators.display_video.GoogleDisplayVideo360UploadLineItemsOperator",
-        "airflow.providers.google.marketing_platform.sensors.display_video.GoogleDisplayVideo360RunQuerySensor",
         "airflow.providers.google.cloud.operators.vertex_ai.generative_model.DeleteExperimentRunOperator",
     }
 
