@@ -18,13 +18,15 @@ from __future__ import annotations
 
 import os
 from importlib.metadata import PackageNotFoundError, metadata
-from unittest import SkipTest, mock
+from unittest import mock
 
 import pytest
+from sqlalchemy import select
+from sqlalchemy.orm import Session
 
 from airflow.models import Connection
 from airflow.secrets.environment_variables import CONN_ENV_PREFIX
-from airflow.utils.session import provide_session
+from airflow.utils.session import NEW_SESSION, provide_session
 
 from tests_common.test_utils.api_fastapi import _check_last_log
 from tests_common.test_utils.asserts import assert_queries_count
@@ -56,7 +58,7 @@ TEST_CONN_TYPE_3 = "test_type_3"
 
 
 @provide_session
-def _create_connection(session) -> None:
+def _create_connection(team_name: str | None = None, session: Session = NEW_SESSION) -> None:
     connection_model = Connection(
         conn_id=TEST_CONN_ID,
         conn_type=TEST_CONN_TYPE,
@@ -64,13 +66,14 @@ def _create_connection(session) -> None:
         host=TEST_CONN_HOST,
         port=TEST_CONN_PORT,
         login=TEST_CONN_LOGIN,
+        team_name=team_name,
     )
     session.add(connection_model)
 
 
 @provide_session
-def _create_connections(session) -> None:
-    _create_connection(session)
+def _create_connections(session: Session = NEW_SESSION) -> None:
+    _create_connection(session=session)
     connection_model_2 = Connection(
         conn_id=TEST_CONN_ID_2,
         conn_type=TEST_CONN_TYPE_2,
@@ -92,8 +95,8 @@ class TestConnectionEndpoint:
     def teardown_method(self) -> None:
         clear_db_connections()
 
-    def create_connection(self):
-        _create_connection()
+    def create_connection(self, team_name: str | None = None):
+        _create_connection(team_name=team_name)
 
     def create_connections(self):
         _create_connections()
@@ -102,11 +105,11 @@ class TestConnectionEndpoint:
 class TestDeleteConnection(TestConnectionEndpoint):
     def test_delete_should_respond_204(self, test_client, session):
         self.create_connection()
-        conns = session.query(Connection).all()
+        conns = session.scalars(select(Connection)).all()
         assert len(conns) == 1
         response = test_client.delete(f"/connections/{TEST_CONN_ID}")
         assert response.status_code == 204
-        connection = session.query(Connection).all()
+        connection = session.scalars(select(Connection)).all()
         assert len(connection) == 0
         _check_last_log(session, dag_id=None, event="delete_connection", logical_date=None)
 
@@ -126,13 +129,14 @@ class TestDeleteConnection(TestConnectionEndpoint):
 
 
 class TestGetConnection(TestConnectionEndpoint):
-    def test_get_should_respond_200(self, test_client, session):
-        self.create_connection()
+    def test_get_should_respond_200(self, test_client, testing_team, session):
+        self.create_connection(team_name=testing_team.name)
         response = test_client.get(f"/connections/{TEST_CONN_ID}")
         assert response.status_code == 200
         body = response.json()
         assert body["connection_id"] == TEST_CONN_ID
         assert body["conn_type"] == TEST_CONN_TYPE
+        assert body["team_name"] == testing_team.name
 
     def test_should_respond_401(self, unauthenticated_test_client):
         response = unauthenticated_test_client.get(f"/connections/{TEST_CONN_ID}")
@@ -150,7 +154,7 @@ class TestGetConnection(TestConnectionEndpoint):
 
     def test_get_should_respond_200_with_extra(self, test_client, session):
         self.create_connection()
-        connection = session.query(Connection).first()
+        connection = session.scalars(select(Connection)).first()
         connection.extra = '{"extra_key": "extra_value"}'
         session.commit()
         response = test_client.get(f"/connections/{TEST_CONN_ID}")
@@ -163,7 +167,7 @@ class TestGetConnection(TestConnectionEndpoint):
     @pytest.mark.enable_redact
     def test_get_should_respond_200_with_extra_redacted(self, test_client, session):
         self.create_connection()
-        connection = session.query(Connection).first()
+        connection = session.scalars(select(Connection)).first()
         connection.extra = '{"password": "test-password"}'
         session.commit()
         response = test_client.get(f"/connections/{TEST_CONN_ID}")
@@ -192,7 +196,7 @@ class TestGetConnection(TestConnectionEndpoint):
 
 class TestGetConnections(TestConnectionEndpoint):
     @pytest.mark.parametrize(
-        "query_params, expected_total_entries, expected_ids",
+        ("query_params", "expected_total_entries", "expected_ids"),
         [
             # Filters
             ({}, 2, [TEST_CONN_ID, TEST_CONN_ID_2]),
@@ -236,6 +240,20 @@ class TestGetConnections(TestConnectionEndpoint):
         response = unauthorized_test_client.get("/connections", params={})
         assert response.status_code == 403
 
+    @mock.patch(
+        "airflow.api_fastapi.auth.managers.base_auth_manager.BaseAuthManager.get_authorized_connections"
+    )
+    def test_should_call_get_authorized_connections(self, mock_get_authorized_connections, test_client):
+        self.create_connections()
+        mock_get_authorized_connections.return_value = {TEST_CONN_ID}
+        response = test_client.get("/connections")
+        mock_get_authorized_connections.assert_called_once_with(user=mock.ANY, method="GET")
+        assert response.status_code == 200
+        body = response.json()
+
+        assert body["total_entries"] == 1
+        assert [connection["connection_id"] for connection in body["connections"]] == [TEST_CONN_ID]
+
 
 class TestPostConnection(TestConnectionEndpoint):
     @pytest.mark.parametrize(
@@ -260,10 +278,29 @@ class TestPostConnection(TestConnectionEndpoint):
     def test_post_should_respond_201(self, test_client, session, body):
         response = test_client.post("/connections", json=body)
         assert response.status_code == 201
-        connection = session.query(Connection).all()
+        connection = session.scalars(select(Connection)).all()
         assert len(connection) == 1
         _check_last_log(session, dag_id=None, event="post_connection", logical_date=None)
 
+    def test_post_should_respond_201_with_team(self, test_client, session, testing_team):
+        response = test_client.post(
+            "/connections",
+            json={"connection_id": TEST_CONN_ID, "conn_type": TEST_CONN_TYPE, "team_name": testing_team.name},
+        )
+        assert response.status_code == 201
+        assert response.json() == {
+            "connection_id": TEST_CONN_ID,
+            "conn_type": TEST_CONN_TYPE,
+            "description": None,
+            "extra": None,
+            "host": None,
+            "login": None,
+            "password": None,
+            "port": None,
+            "schema": None,
+            "team_name": testing_team.name,
+        }
+
     def test_should_respond_401(self, unauthenticated_test_client):
         response = unauthenticated_test_client.post("/connections", json={})
         assert response.status_code == 401
@@ -315,7 +352,7 @@ class TestPostConnection(TestConnectionEndpoint):
 
     @pytest.mark.enable_redact
     @pytest.mark.parametrize(
-        "body, expected_response",
+        ("body", "expected_response"),
         [
             (
                 {"connection_id": TEST_CONN_ID, "conn_type": TEST_CONN_TYPE, "password": "test-password"},
@@ -329,6 +366,7 @@ class TestPostConnection(TestConnectionEndpoint):
                     "password": "***",
                     "port": None,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
             (
@@ -343,6 +381,7 @@ class TestPostConnection(TestConnectionEndpoint):
                     "password": "***",
                     "port": None,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
             (
@@ -362,6 +401,7 @@ class TestPostConnection(TestConnectionEndpoint):
                     "password": "***",
                     "port": None,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
         ],
@@ -375,7 +415,7 @@ class TestPostConnection(TestConnectionEndpoint):
 
 class TestPatchConnection(TestConnectionEndpoint):
     @pytest.mark.parametrize(
-        "body, expected_result",
+        ("body", "expected_result"),
         [
             (
                 {"connection_id": TEST_CONN_ID, "conn_type": "new_type", "extra": '{"key": "var"}'},
@@ -389,6 +429,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": None,
                     "port": TEST_CONN_PORT,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
             (
@@ -403,6 +444,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": None,
                     "port": TEST_CONN_PORT,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
             (
@@ -422,6 +464,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": None,
                     "port": 80,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
             (
@@ -436,6 +479,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": None,
                     "port": TEST_CONN_PORT,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
             (
@@ -450,6 +494,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": None,
                     "port": 80,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
             (
@@ -470,6 +515,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": "test_password_patch",
                     "port": 80,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
             (
@@ -491,6 +537,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": None,
                     "port": 80,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
             (
@@ -510,6 +557,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": None,
                     "port": TEST_CONN_PORT,
                     "schema": "http_patch",
+                    "team_name": None,
                 },
             ),
             (
@@ -534,11 +582,11 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": None,
                     "port": None,
                     "schema": None,
+                    "team_name": None,
                 },
             ),
         ],
     )
-    @provide_session
     def test_patch_should_respond_200(
         self, test_client, body: dict[str, str], expected_result: dict[str, str], session
     ):
@@ -550,6 +598,29 @@ class TestPatchConnection(TestConnectionEndpoint):
 
         assert response.json() == expected_result
 
+    def test_patch_with_team_should_respond_200(self, test_client, testing_team, session):
+        self.create_connection()
+
+        response = test_client.patch(
+            f"/connections/{TEST_CONN_ID}",
+            json={"connection_id": TEST_CONN_ID, "conn_type": "new_type", "team_name": testing_team.name},
+        )
+        assert response.status_code == 200
+        _check_last_log(session, dag_id=None, event="patch_connection", logical_date=None)
+
+        assert response.json() == {
+            "conn_type": "new_type",
+            "connection_id": TEST_CONN_ID,
+            "description": TEST_CONN_DESCRIPTION,
+            "extra": None,
+            "host": TEST_CONN_HOST,
+            "login": TEST_CONN_LOGIN,
+            "password": None,
+            "port": TEST_CONN_PORT,
+            "schema": None,
+            "team_name": testing_team.name,
+        }
+
     def test_should_respond_401(self, unauthenticated_test_client):
         response = unauthenticated_test_client.patch(f"/connections/{TEST_CONN_ID}", json={})
         assert response.status_code == 401
@@ -559,7 +630,7 @@ class TestPatchConnection(TestConnectionEndpoint):
         assert response.status_code == 403
 
     @pytest.mark.parametrize(
-        "body, updated_connection, update_mask",
+        ("body", "updated_connection", "update_mask"),
         [
             (
                 {
@@ -579,6 +650,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "schema": None,
                     "password": None,
                     "description": TEST_CONN_DESCRIPTION,
+                    "team_name": None,
                 },
                 {"update_mask": ["login", "port"]},
             ),
@@ -600,6 +672,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "schema": None,
                     "password": None,
                     "description": TEST_CONN_DESCRIPTION,
+                    "team_name": None,
                 },
                 {"update_mask": ["login", "port"]},
             ),
@@ -615,6 +688,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "schema": None,
                     "password": None,
                     "description": TEST_CONN_DESCRIPTION,
+                    "team_name": None,
                 },
                 {"update_mask": ["host"]},
             ),
@@ -635,6 +709,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "schema": None,
                     "password": None,
                     "description": TEST_CONN_DESCRIPTION,
+                    "team_name": None,
                 },
                 {"update_mask": ["host", "port"]},
             ),
@@ -650,6 +725,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "schema": None,
                     "password": None,
                     "description": TEST_CONN_DESCRIPTION,
+                    "team_name": None,
                 },
                 {"update_mask": ["login"]},
             ),
@@ -670,6 +746,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": None,
                     "schema": None,
                     "description": TEST_CONN_DESCRIPTION,
+                    "team_name": None,
                 },
                 {"update_mask": ["host"]},
             ),
@@ -692,6 +769,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": None,
                     "schema": "new_schema",
                     "description": TEST_CONN_DESCRIPTION,
+                    "team_name": None,
                 },
                 {"update_mask": ["schema", "extra"]},
             ),
@@ -703,7 +781,7 @@ class TestPatchConnection(TestConnectionEndpoint):
         self.create_connection()
         response = test_client.patch(f"/connections/{TEST_CONN_ID}", json=body, params=update_mask)
         assert response.status_code == 200
-        connection = session.query(Connection).filter_by(conn_id=TEST_CONN_ID).first()
+        connection = session.scalars(select(Connection).where(Connection.conn_id == TEST_CONN_ID)).first()
         assert connection.password is None
         assert response.json() == updated_connection
 
@@ -790,7 +868,7 @@ class TestPatchConnection(TestConnectionEndpoint):
 
     @pytest.mark.enable_redact
     @pytest.mark.parametrize(
-        "body, expected_response, update_mask",
+        ("body", "expected_response", "update_mask"),
         [
             (
                 {"connection_id": TEST_CONN_ID, "conn_type": TEST_CONN_TYPE, "password": "test-password"},
@@ -804,6 +882,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": "***",
                     "port": 8080,
                     "schema": None,
+                    "team_name": None,
                 },
                 {"update_mask": ["password"]},
             ),
@@ -819,6 +898,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": "***",
                     "port": 8080,
                     "schema": None,
+                    "team_name": None,
                 },
                 {"update_mask": ["password"]},
             ),
@@ -839,6 +919,7 @@ class TestPatchConnection(TestConnectionEndpoint):
                     "password": "***",
                     "port": 8080,
                     "schema": None,
+                    "team_name": None,
                 },
                 {"update_mask": ["password", "extra"]},
             ),
@@ -859,11 +940,11 @@ class TestConnection(TestConnectionEndpoint):
         try:
             metadata("apache-airflow-providers-sqlite")
         except PackageNotFoundError:
-            raise SkipTest("The SQlite distribution package is not installed.")
+            pytest.skip("The SQlite distribution package is not installed.")
 
     @mock.patch.dict(os.environ, {"AIRFLOW__CORE__TEST_CONNECTION": "Enabled"})
     @pytest.mark.parametrize(
-        "body, message",
+        ("body", "message"),
         [
             ({"connection_id": TEST_CONN_ID, "conn_type": "sqlite"}, "Connection successfully tested"),
             (
@@ -945,7 +1026,7 @@ class TestCreateDefaultConnections(TestConnectionEndpoint):
 
 class TestBulkConnections(TestConnectionEndpoint):
     @pytest.mark.parametrize(
-        "actions, expected_results",
+        ("actions", "expected_results"),
         [
             pytest.param(
                 {
@@ -1128,6 +1209,26 @@ class TestBulkConnections(TestConnectionEndpoint):
                 },
                 id="test_update_with_fail",
             ),
+            pytest.param(
+                {
+                    "actions": [
+                        {
+                            "action": "update",
+                            "entities": [
+                                {
+                                    "connection_id": TEST_CONN_ID,
+                                    "conn_type": TEST_CONN_TYPE,
+                                    "description": "updated_description",
+                                }
+                            ],
+                            "update_mask": ["description"],
+                            "action_on_non_existence": "fail",
+                        }
+                    ]
+                },
+                {"update": {"success": [TEST_CONN_ID], "errors": []}},
+                id="test_connection_update_with_valid_update_mask",
+            ),
             pytest.param(
                 {
                     "actions": [
@@ -1233,6 +1334,34 @@ class TestBulkConnections(TestConnectionEndpoint):
                 },
                 id="test_create_update_delete",
             ),
+            pytest.param(
+                {
+                    "actions": [
+                        {
+                            "action": "update",
+                            "entities": [
+                                {
+                                    "connection_id": TEST_CONN_ID,
+                                    "conn_type": TEST_CONN_TYPE,
+                                    "description": "updated_description",
+                                }
+                            ],
+                            "update_mask": ["description"],
+                            "action_on_non_existence": "fail",
+                        },
+                        {
+                            "action": "delete",
+                            "entities": [TEST_CONN_ID],
+                            "action_on_non_existence": "fail",
+                        },
+                    ]
+                },
+                {
+                    "update": {"success": [TEST_CONN_ID], "errors": []},
+                    "delete": {"success": [TEST_CONN_ID], "errors": []},
+                },
+                id="test_connection_create_update_delete_with_update_mask",
+            ),
         ],
     )
     def test_bulk_connections(self, test_client, actions, expected_results, session):
@@ -1271,12 +1400,12 @@ class TestPostConnectionExtraBackwardCompatibility(TestConnectionEndpoint):
         response = test_client.post("/connections", json=body)
         assert response.status_code == 201
 
-        connection = session.query(Connection).filter_by(conn_id=TEST_CONN_ID).first()
+        connection = session.scalars(select(Connection).where(Connection.conn_id == TEST_CONN_ID)).first()
         assert connection is not None
         assert connection.extra == "{}"  # Backward compatibility: treat "" as empty JSON object
 
     @pytest.mark.parametrize(
-        "extra, expected_error_message",
+        ("extra", "expected_error_message"),
         [
             ("[1,2,3]", "Expected JSON object in `extra` field, got non-dict JSON"),
             ("some_string", "Encountered non-JSON in `extra` field"),
