@@ -29,7 +29,8 @@ import jinja2
 import pytest
 import structlog
 
-from airflow.sdk import task as task_decorator
+from airflow.sdk import DAG, Label, TaskGroup, task as task_decorator
+from airflow.sdk._shared.secrets_masker import _secrets_masker, mask_secret
 from airflow.sdk.bases.operator import (
     BaseOperator,
     BaseOperatorMeta,
@@ -38,11 +39,8 @@ from airflow.sdk.bases.operator import (
     chain_linear,
     cross_downstream,
 )
-from airflow.sdk.definitions.dag import DAG
-from airflow.sdk.definitions.edges import Label
-from airflow.sdk.definitions.taskgroup import TaskGroup
+from airflow.sdk.definitions.param import ParamsDict
 from airflow.sdk.definitions.template import literal
-from airflow.task.priority_strategy import _DownstreamPriorityWeightStrategy, _UpstreamPriorityWeightStrategy
 
 DEFAULT_DATE = datetime(2016, 1, 1, tzinfo=timezone.utc)
 
@@ -240,6 +238,23 @@ class TestBaseOperator:
                 illegal_argument_1234="hello?",
             )
 
+    @mock.patch("airflow.sdk.bases.operator.redact")
+    def test_illegal_args_with_secrets(self, mock_redact):
+        """
+        Tests that operators on illegal arguments with secrets are correctly masked.
+        """
+        secret = "secretP4ssw0rd!"
+        mock_redact.side_effect = ["***"]
+
+        msg = r"Invalid arguments were passed to BaseOperator"
+        with pytest.raises(TypeError, match=msg) as exc_info:
+            BaseOperator(
+                task_id="test_illegal_args",
+                secret_argument=secret,
+            )
+        assert "***" in str(exc_info.value)
+        assert secret not in str(exc_info.value)
+
     def test_invalid_type_for_default_arg(self):
         error_msg = "'max_active_tis_per_dag' for task 'test' expects <class 'int'>, got <class 'str'> with value 'not_an_int'"
         with pytest.raises(TypeError, match=error_msg):
@@ -252,29 +267,12 @@ class TestBaseOperator:
 
     def test_weight_rule_default(self):
         op = BaseOperator(task_id="test_task")
-        assert _DownstreamPriorityWeightStrategy() == op.weight_rule
+        assert op.weight_rule == "downstream"
 
     def test_weight_rule_override(self):
-        op = BaseOperator(task_id="test_task", weight_rule="upstream")
-        assert _UpstreamPriorityWeightStrategy() == op.weight_rule
-
-    def test_dag_task_invalid_weight_rule(self):
-        # Test if we enter an invalid weight rule
-        with pytest.raises(ValueError):
-            BaseOperator(task_id="should_fail", weight_rule="no rule")
-
-    def test_dag_task_not_registered_weight_strategy(self):
-        from airflow.task.priority_strategy import PriorityWeightStrategy
-
-        class NotRegisteredPriorityWeightStrategy(PriorityWeightStrategy):
-            def get_weight(self, ti):
-                return 99
-
-        with pytest.raises(ValueError, match="Unknown priority strategy"):
-            BaseOperator(
-                task_id="empty_task",
-                weight_rule=NotRegisteredPriorityWeightStrategy(),
-            )
+        whatever_value = object()
+        op = BaseOperator(task_id="test_task", weight_rule=whatever_value)
+        assert op.weight_rule is whatever_value
 
     def test_db_safe_priority(self):
         """Test the db_safe_priority function."""
@@ -479,10 +477,12 @@ class TestBaseOperator:
         [label1, label2] = [Label(label=f"label{i}") for i in range(1, 3)]
         [op1, op2, op3, op4, op5] = [BaseOperator(task_id=f"t{i}", dag=dag) for i in range(1, 6)]
 
-        with pytest.raises(ValueError):
+        CHAIN_NOT_SUPPORTED = "Chain not supported for different length Iterable. Got {} and {}."
+
+        with pytest.raises(ValueError, match=CHAIN_NOT_SUPPORTED.format(2, 3)):
             chain([op1, op2], [op3, op4, op5])
 
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match=CHAIN_NOT_SUPPORTED.format(3, 2)):
             chain([op1, op2, op3], [label1, label2])
 
         # Begin test for `XComArgs` with `EdgeModifiers`
@@ -492,16 +492,16 @@ class TestBaseOperator:
             for i in range(1, 6)
         ]
 
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match=CHAIN_NOT_SUPPORTED.format(2, 3)):
             chain([xop1, xop2], [xop3, xop4, xop5])
 
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match=CHAIN_NOT_SUPPORTED.format(3, 2)):
             chain([xop1, xop2, xop3], [label1, label2])
 
         # Begin test for `TaskGroups`
         [tg1, tg2, tg3, tg4, tg5] = [TaskGroup(group_id=f"tg{i}", dag=dag) for i in range(1, 6)]
 
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match=CHAIN_NOT_SUPPORTED.format(2, 3)):
             chain([tg1, tg2], [tg3, tg4, tg5])
 
     def test_set_xcomargs_dependencies_works_recursively(self):
@@ -526,7 +526,10 @@ class TestBaseOperator:
 
     def test_set_xcomargs_dependencies_error_when_outside_dag(self):
         op1 = BaseOperator(task_id="op1")
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError,
+            match=r"Tried to create relationships between tasks that don't have Dags yet. Set the Dag for at least one task and try again: \[<Task\(MockOperator\): op2>, <Task\(BaseOperator\): op1>\]",
+        ):
             MockOperator(task_id="op2", arg1=op1.output)
 
     def test_cannot_change_dag(self):
@@ -706,7 +709,7 @@ class TestBaseOperator:
         task.render_template_fields({})
         assert task.arg2 == "foo_barbarbar"
 
-    @pytest.mark.parametrize(("content",), [(object(),), (uuid.uuid4(),)])
+    @pytest.mark.parametrize("content", [object(), uuid.uuid4()])
     def test_render_template_fields_no_change(self, content):
         """Tests if non-templatable types remain unchanged."""
         task = BaseOperator(task_id="op1")
@@ -762,6 +765,36 @@ class TestBaseOperator:
         task.render_template_fields(context={"foo": "whatever", "bar": "whatever"})
         assert mock_jinja_env.call_count == 1
 
+    def test_params_source(self):
+        # Test bug when copying an operator attached to a Dag
+        with DAG(
+            "dag0",
+            params=ParamsDict(
+                {
+                    "param from Dag": "value1",
+                    "overwritten by task": "value 2",
+                }
+            ),
+            schedule=None,
+            start_date=DEFAULT_DATE,
+        ):
+            op1 = MockOperator(
+                task_id="task1",
+                params=ParamsDict(
+                    {
+                        "overwritten by task": "value 3",
+                        "param from task": "value 4",
+                    }
+                ),
+            )
+
+        for key, expected_source in (
+            ("param from Dag", "dag"),
+            ("overwritten by task", "task"),
+            ("param from task", "task"),
+        ):
+            assert op1.params.get_param(key).source == expected_source
+
     def test_deepcopy(self):
         # Test bug when copying an operator attached to a Dag
         with DAG("dag0", schedule=None, start_date=DEFAULT_DATE) as dag:
@@ -784,7 +817,7 @@ class TestBaseOperator:
             pass
 
         # The following throws an exception if metaclass breaks MRO:
-        #   airflow.exceptions.AirflowException: Invalid arguments were passed to Branch (task_id: test). Invalid arguments were:
+        #   airflow.sdk.exceptions.AirflowException: Invalid arguments were passed to Branch (task_id: test). Invalid arguments were:
         #   **kwargs: {'sql': 'sql', 'follow_task_ids_if_true': ['x'], 'follow_task_ids_if_false': ['y']}
         op = Branch(
             task_id="test",
@@ -918,6 +951,30 @@ def test_render_template_fields_logging(
         assert not_expected_log not in caplog.text
 
 
+@pytest.mark.enable_redact
+def test_render_template_fields_secret_masking(caplog):
+    """Test that sensitive values are masked in Jinja template rendering exceptions."""
+    masker = _secrets_masker()
+    masker.reset_masker()
+
+    masker.sensitive_variables_fields = ["password", "secret", "token"]
+
+    mask_secret("mysecretpassword", "password")
+
+    task = MockOperator(task_id="op1", arg1="{{ password + 1 }}")
+    context = {"password": "mysecretpassword"}
+
+    with (
+        pytest.raises(TypeError),
+        caplog.at_level(logging.ERROR, logger="airflow.sdk.definitions.templater"),
+    ):
+        task.render_template_fields(context=context)
+
+    assert "mysecretpassword" not in caplog.text
+    assert "Template: '{{ password + 1 }}'" in caplog.text
+    assert "Exception rendering Jinja template for task 'op1', field 'arg1'" in caplog.text
+
+
 class HelloWorldOperator(BaseOperator):
     log = structlog.get_logger(__name__)
 
